package types

import (
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
)

const TypeMsgNewAuction = "new_auction"

var _ sdk.Msg = &MsgNewAuction{}

func NewMsgNewAuction(
	// auction params
	creator string, name string, description string, ceiling string, denom string, leaseEnd uint64, vProviders []string,
	// hardware specs
	gpus []string, cpuCores uint64, cpuArch CpuArchitecture, memMb uint64, storageGb uint64,
) *MsgNewAuction {
	return &MsgNewAuction{
		Creator: creator,

		// creation params
		Name:              name,
		Description:       description,
		Ceiling:           ceiling,
		Denom:             denom,
		LeaseEnd:          leaseEnd,
		VerifiedProviders: vProviders,

		// hardware
		Gpus:      gpus,
		CpuCores:  cpuCores,
		CpuArch:   cpuArch,
		MemMb:     memMb,
		StorageGb: storageGb,
	}
}

func (msg *MsgNewAuction) Route() string {
	return RouterKey
}

func (msg *MsgNewAuction) Type() string {
	return TypeMsgNewAuction
}

func (msg *MsgNewAuction) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgNewAuction) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgNewAuction) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}
